;****************************************************************
;* Script to generate a netcdf file containing a user specified *
;* temperature tendency for input to the CAM dry dynamical core *
;*                                                              *
;* Input:							*
;*   filein = File containing the user specified temperature    *
;*            tendency                                          *
;*            Assumes this contains the variable q(lon,lat,pre) * 
;              = the temperature tendency in K/s                *
;*   res = horizontal resolution of model simulation            *
;*   nlev = number of vertical levels in model simulation       *
;* Output:
;*   fileout = file containing temperature tendency interpolated*
;*             onto the CAM grid and in the correct format for  *
;*             input to the model                               *
;****************************************************************
; !!WARNING - this will not extrapolate to lons, lats and pre's *
; that are outside of the domain of the input file so make sure *
; the input covers the range of lons, lats and pres required for*
; the cam grid
;****************************************************************
;*  Isla Simpson 18th April 2018                                *
;****************************************************************

; ----read in input temperature tendency
filein="./input/myqpert.nc"

; ----specify output filename
setfileoption("nc","Format","LargeFile")
fileout = "./output/qpertinput_T42.nc"
system("/bin/rm -f "+fileout) ; remove if exists
fout = addfile(fileout,"c") ; open output file

; ----vertical level options (uncomment desied resolution
 nlev=30
; nlev=40
; nlev=60
; nlev=80
 load "./levels.ncl"

; ----horizontal resolution options
  res = "64x128" 		;Eulerian dycore at T42
; res = "128x256"               ; "          "   "  T85
;res = "256x512"  	        ; "          "   "  T255
;res = "512x1024"               ; "          "   "  T511 (?) 
;res = "f19" 			;FV dycore at 2 deg resolution
load "./gaus_horizontal_grid.ncl"

  P0 =  100000.0d0
  P0!0="ncl_scalar"
  P0@long_name = "reference pressure"
  P0@units = "Pa"

;-- Read in input q profile on arbitrary horizontal and vertical grid
  infile=addfile(filein,"r")
  lonin=infile->lon
  latin=infile->lat
  prein=infile->pre
  qin=infile->q(:,:,:)

;-- horizontal interpolation onto gaussian grid
  qnew = linint2(lonin,latin,qin,False,lon,lat,0)

;--vetrical interpolation onto model levels (assuming ps=1000)

   qnew2=new( (/1,dimsizes(lev),dimsizes(lat),dimsizes(lon)/),"double")
   qnew2=int2p_n(prein,qnew,lev,2,0)
   qnew2!0 = "time"
   qnew2!1 = "lev"
   qnew2!2 = "lat"
   qnew2!3 = "lon"

;-- Time coordinate
  ntime=1
  time = (/0.d0/)
  time!0 = "time"
  time@long_name = "time"
  time@units = "days since 1979-01-01 00:00:00"
  time@calendar = "noleap"
  time@bounds = "time_bnds"

  date = (/19790101/)
  date!0="time"
  date@long_name = "current date (YYYYMMDD)"

  datesec = (/0/)
  datesec!0="time"
  datesec@long_name = "current seconds of current date"

;--output interpolated q
  setfileoption(fout,"DefineMode",True)

;--global attributes
  fAtt               = True            ; assign file attributes
  fAtt@title         = "User specified relaxation temperature profile"
  fAtt@source_file   = "ncl script"
  fAtt@Conventions   = "None"
  fAtt@creation_date = systemfunc ("date")
  fileattdef( fout, fAtt )             ; copy file attributes

;--pre-define coordinate variables
  dimNames = (/"time","lev","lat","lon"/)
  dimSizes= (/-1, dimsizes(lev), dimsizes(lat),dimsizes(lon)/)
  dimUnlim = (/True, False, False, False/)
  filedimdef(fout,dimNames,dimSizes,dimUnlim)

;--pre-define output variables
  filevardef(fout,      "time", typeof(time),   getvardims(time))
  filevardef(fout,        "date" ,typeof(date)          ,getvardims(date))
  filevardef(fout,     "datesec" ,typeof(datesec)       ,getvardims(datesec))

  filevardef(fout,         "lev" ,typeof(lev)           ,getvardims(lev) )
  filevardef(fout,        "hyam" ,typeof(hyam)          ,getvardims(hyam))
  filevardef(fout,        "hybm" ,typeof(hybm)          ,getvardims(hybm))
  filevardef(fout,          "P0" ,typeof(P0)            ,getvardims(P0))

  filevardef(fout,         "lat" ,typeof(lat)           ,getvardims(lat))
  filevardef(fout,         "lon" ,typeof(lon)           ,getvardims(lon))
;  filevardef(fout,          "gw" ,typeof(gw)            ,getvardims(gw))

  filevardef(fout,          "q" ,typeof(qnew2)            ,getvardims(qnew2))

; copy attributes
  filevarattdef(fout,"time" ,time)                ; copy time attributes
  filevarattdef(fout,"date" ,date)                ; copy   date attributes
  filevarattdef(fout,"datesec",datesec)           ; copy   datesec attributes

  filevarattdef(fout,"lev"  ,lev)                 ; copy  lev attributes
  filevarattdef(fout,"hyam" ,hyam)                ; copy   hyam attributes
  filevarattdef(fout,"hybm" ,hybm)                ; copy   hybm attributes
  filevarattdef(fout,"P0"   ,P0)                  ; copy   P0 attributes

  filevarattdef(fout,"lat"  ,lat)                 ; copy  lat attributes
  filevarattdef(fout,"lon"  ,lon)                 ; copy  lon attributes

  eulstrings=( (/"64x125","128x256","256x512","512x1024"/) )
  if (.not.ismissing(str_match(eulstrings,res))) then 
    filevarattdef(fout,"gw"   ,gw)                  ; copy   gw attributes
  end if

  filevarattdef(fout,"q"  ,qnew2)                  ; copy    q attributes

  setfileoption(fout,"DefineMode",False)

  fout->time         = (/time/)
  fout->date         = (/date/)
  fout->datesec      = (/datesec/)

  fout->lev          = (/lev/)
  fout->hyam         = (/hyam/)
  fout->hybm         = (/hybm/)
  fout->P0           = (/P0/)

  fout->lat          = (/lat/)
  fout->lon          = (/lon/)
  if (.not.ismissing(str_match(eulstrings,res))) then 
   fout->gw	     = (/gw/)
  end if
  fout->q            = (/qnew2/)
