#define MODHS 1
#undef MODHS
module held_suarez_cam
! ----------------------------------------------------------------- 
! Modified to read in an additional temperature tendency from file
! Mods are denoted by 
! 
! QPERT-READ
!   blah blah
! END QPERT-READ
! Isla Simpson 21st April 2018
! -----------------------------------------------------------------

  use shr_kind_mod, only: r8 => shr_kind_r8

!QPERT-READ
!  use ppgrid,       only: pcols, pver
  use ppgrid,  only: pcols, pver, begchunk, endchunk
  use spmd_utils , only: mpicom,mstrid=>masterprocid, mpi_integer, mpi_real8, &
                        mpi_logical, mpi_character, masterproc
  use cam_abortutils , only: endrun
  use cam_logfile, only : iulog
  !END QPERT-READ



  implicit none
  private
  save

  public :: held_suarez_init, held_suarez_tend

  ! QPERT-READ
  public :: qpertread_readnl
  ! END QPERT-READ

  real(r8), parameter :: efoldf  =  1._r8  ! efolding time for wind dissipation
  real(r8), parameter :: efolda  = 40._r8  ! efolding time for T dissipation
  real(r8), parameter :: efolds  =  4._r8  ! efolding time for T dissipation
  real(r8), parameter :: sigmab  =  0.7_r8 ! threshold sigma level
  real(r8), parameter :: t00     = 200._r8 ! minimum reference temperature
  real(r8), parameter :: kf      = 1._r8/(86400._r8*efoldf) ! 1./efolding_time for wind dissipation

  real(r8), parameter :: onemsig = 1._r8 - sigmab ! 1. - sigma_reference

  real(r8), parameter :: ka      = 1._r8/(86400._r8 * efolda) ! 1./efolding_time for temperature diss.
  real(r8), parameter :: ks      = 1._r8/(86400._r8 * efolds)

  !QPERT-READ
  character(len=256) :: qpertfile
  logical :: qpertfromfile
  real(r8), allocatable :: qpert(:,:,:) ! additional temperature tendency
  !END QPERT-READ

!======================================================================= 
contains
!======================================================================= 

  subroutine held_suarez_init(pbuf2d)
    use physics_buffer,     only: physics_buffer_desc
    use cam_history,        only: addfld, add_default
    use physconst,          only: cappa, cpair
    use ref_pres,           only: pref_mid_norm, psurf_ref
    use held_suarez,        only: held_suarez_1994_init

    !QPERT-READ
    use error_messages, only: alloc_err
    !END QPERT-READ

    type(physics_buffer_desc), pointer :: pbuf2d(:,:)

    !QPERT-READ
    integer :: istat
    !END QPERT-READ

    ! Set model constant values
    call held_suarez_1994_init(cappa, cpair, psurf_ref, pref_mid_norm)

    !QPERT-READ
    allocate(qpert(pcols,pver,begchunk:endchunk), stat=istat)
    call alloc_err(istat,'UPDATE_QPERT','qpert',pcols*pver*(endchunk-begchunk))
    qpert(:pcols,:pver,begchunk:endchunk)=0._r8

    if (qpertfromfile) then
     call update_qpert(qpertfile)
    end if
    !END QPERT_READ 



    ! This field is added by radiation when full physics is used
    call addfld('QRS', (/ 'lev' /), 'A', 'K/s', &
         'Temperature tendency associated with the relaxation toward the equilibrium temperature profile')
    call add_default('QRS', 1, ' ')

    !QPERT-READ
    ! Output the additional temperature tendency read in from file
    call  addfld('QPERT', (/ 'lev' /),'A','K/s',&
      'Additional imposed temperature tendency')
    call add_default('QPERT', 1, ' ')
    !END QPERT-READ

 end subroutine held_suarez_init

  subroutine held_suarez_tend(state, ptend, ztodt)
    !----------------------------------------------------------------------- 
    ! 
    ! Purpose: 
    !  algorithm 1: Held/Suarez IDEALIZED physics
    !  algorithm 2: Held/Suarez IDEALIZED physics (Williamson modified stratosphere
    !  algorithm 3: Held/Suarez IDEALIZED physics (Lin/Williamson modified strato/meso-sphere
    !
    ! Author: J. Olson
    ! 
    !-----------------------------------------------------------------------
    use physconst,          only: cpairv
    use phys_grid,          only: get_rlat_all_p
    use physics_types,      only: physics_state, physics_ptend
    use physics_types,      only: physics_ptend_init
    use cam_abortutils,     only: endrun
    use cam_history,        only: outfld
    use held_suarez,        only: held_suarez_1994

    !
    ! Input arguments
    !
    type(physics_state), intent(inout) :: state
    real(r8),            intent(in)    :: ztodt            ! Two times model timestep (2 delta-t)
                                                           !
                                                           ! Output argument
                                                           !
    type(physics_ptend), intent(out)   :: ptend            ! Package tendencies
                                                           !
    !---------------------------Local workspace-----------------------------

    integer                            :: lchnk            ! chunk identifier
    integer                            :: ncol             ! number of atmospheric columns

    real(r8)                           :: clat(pcols)      ! latitudes(radians) for columns
    real(r8)                           :: pmid(pcols,pver) ! mid-point pressure
    integer                            :: i, k             ! Longitude, level indices

    !QPERT-READ
    real(r8)                           :: qpertlchnk(pcols,pver) ! heating perturbation of current chunk 
    !END QPERT-READ


    !
    !-----------------------------------------------------------------------
    !

    lchnk = state%lchnk
    ncol  = state%ncol

    call get_rlat_all_p(lchnk, ncol, clat)
    do k = 1, pver
      do i = 1, ncol
        pmid(i,k) = state%pmid(i,k)
        !QPERT-READ
        if (qpertfromfile) then
          qpertlchnk(i,k)=qpert(i,k,lchnk)
        end if
        !END QPERT-READ
      end do
    end do

    ! initialize individual parameterization tendencies
    call physics_ptend_init(ptend, state%psetcols, 'held_suarez', ls=.true., lu=.true., lv=.true.)

    !QPERT-READ
    !call held_suarez_1994(pcols, ncol, clat, state%pmid, &
    !     state%u, state%v, state%t, ptend%u, ptend%v, ptend%s)
    if (qpertfromfile) then
      call held_suarez_1994(pcols, ncol, clat, pmid, &
         state%u, state%v, state%t, ptend%u, ptend%v, ptend%s, qpertlchnk)
    else
      call held_suarez_1994(pcols, ncol, clat, state%pmid, &
         state%u, state%v, state%t, ptend%u, ptend%v, ptend%s)
    end if
    !END QPERT-READ



    ! Note, we assume that there are no subcolumns in simple physics
    pmid(:ncol,:) = ptend%s(:ncol, :)/cpairv(:ncol,:,lchnk)
    if (pcols > ncol) then
      pmid(ncol+1:,:) = 0.0_r8
    end if
    call outfld('QRS', pmid, pcols, lchnk)

    !QPERT-READ
    call outfld('QPERT', qpertlchnk, pcols, lchnk)
    !END QPERT-READ


  end subroutine held_suarez_tend

!QPERT-READ
   subroutine qpertread_readnl(nlfile)
  ! -----------------------------------------------------------------------
  ! Read in namelist parameters qpertfromfile and qpertfile
  ! qpertfromfile = controls whether an additional temperature tendency is
  ! imposed
  !             or not. Default = qpertfromfile=.False. --> no additional
  !             tendency 
  ! qpertfile = file containing structure of additional temperature tendency
  ! Isla Simpson (21st Apr 2018)
  ! -----------------------------------------------------------------------
  use namelist_utils  ,only:find_group_name
  use units           ,only:getunit,freeunit
  character(len=*),intent(in)::nlfile
  integer :: unitn, ierr
  character(len=*), parameter :: sub = 'qpertread_readnl'

  namelist /qpertread_nl/ qpertfromfile,qpertfile

  !Set default namelist values
  qpertfromfile=.False.
  qpertfile='randomfile'

  !Read in namelist values
  if (masterproc) then
   unitn = getunit()
   open(unitn,file=trim(nlfile),status='old')
   call find_group_name(unitn,'qpertread_nl',status=ierr)
   if (ierr.eq.0) then
     read(unitn,qpertread_nl,iostat=ierr)
     if (ierr.ne.0) then
       call endrun('qpertread_readnl:: ERROR reading namelist')
     endif
   endif
   close(unitn)
   call freeunit(unitn)
   endif


   call mpi_bcast(qpertfromfile   , 1, mpi_logical, mstrid, mpicom, ierr)
   if (ierr /= 0) call endrun(sub//": FATAL: mpi_bcast: held_suarez_1994")
   call mpi_bcast(qpertfile       , len(qpertfile), mpi_character, mstrid, mpicom, ierr)
   if (ierr /= 0) call endrun(sub//": FATAL: mpi_bcast: held_suarez_1994")

   return
  end subroutine
  !END QPERT-READ

  !QPERT-READ
  !------------------------------------------
  subroutine update_qpert(qpertfilein)
  ! --------------------------------
  ! Read in an additional temperature tendency from a netcdf file 
  ! Isla Simpson 21st April 2018
  ! --------------------------------
  use cam_grid_support, only: cam_grid_get_dim_names, cam_grid_id
  use pio, only: pio_nowrite, file_desc_t, pio_closefile
  use ioFileMod, only: getfil
  use cam_pio_utils, only: cam_pio_openfile
  use ncdio_atm, only: infld

  character(len=*), intent(in) :: qpertfilein
  character(len=8) :: dim1name, dim2name
  integer :: grid_id
  type(file_desc_t), pointer :: ncidqpert
  character(len=256) :: qpertpath
  logical :: found=.false.

  grid_id = cam_grid_id('physgrid')
  call cam_grid_get_dim_names(grid_id, dim1name, dim2name)

  allocate(ncidqpert)
  call getfil(qpertfilein, qpertpath)
  call cam_pio_openfile(ncidqpert, trim(qpertpath), PIO_NOWRITE)
  call infld('q',ncidqpert,dim1name,'lev',dim2name,1,pcols,1,pver, &
    begchunk,endchunk, qpert, found, gridname='physgrid')
  if (.not.found) then
    write(iulog,*) 'QPERT, variable q not found in input file ',qpertfile
    call endrun('QPERT :: Error, variable q not found in input file')
  end if


  call pio_closefile(ncidqpert)
  end subroutine
  !------------------------------------------
  !END QPERT-READ



end module held_suarez_cam
